using Optim
using ForwardDiff

#first thing: include problem definition file
println("Which problem do you want to solve? (R) max return  (V) min variance:")
iRet = 0
iVar = 0
while iRet == 0 && iVar == 0
	s = readline()
	iRet = search(s,'R')
	if iRet == 0 
		iVar = search(s,'V')
		if iVar == 0
			println("Incorrect choice. Try again")
		end
	else
		iVar = 0
	end
end
if iVar != 0
	include("prob_mark_risk.jl")
	println()
	println("Problem is:")
	println("\tmin  x'*Q*x")
	println("\ts.t. x'*R >= sigma")
	println("\t     x'*e  = 1.0")
	println("\t   0 <= x <= 1")
	# ret >= sigma
	sigma_min = 0.374
	sigma_max = 0.696
	#sigma_min = 0.0	   # ret >= sigma
	sigma = sigma_min
	fid = open("minvar.txt","w")
else
	include("prob_mark.jl")
	println()
	println("Problem is:")
	println("\tmin -x'*R")
	println("\ts.t. x'*Q*x <= sigma")
	println("\t     x'*e    = 1.0")
	println("\t     0 <= x <= 1")
	sigma_min = 0.035
	sigma_max = 0.19
	#sigma_max = 1.0    # risk <= sigma
	sigma = sigma_max
	fid = open("maxret.txt","w")
end

nPtoS = 20 # number of problems to solve consequtively
	  # solve nPtoS problems for values of sigma evenly distributed 
	  # betwee sigma_min and sigma_max

#      ret          var
# 6.961566e-01  1.903364e-01 max ret
# 3.736577e-01  3.243245e-02 min var


if nPtoS == 1
	if iVar == 0
		sigmas = [sigma_max] # risk <= sigma
	else
		sigmas = [sigma_min] # ret >= sigma
	end
else
	sigmas = collect(linspace(sigma_min,sigma_max,nPtoS))
end
#sigma = 0.5 # return >= sigma

for vsigma in sigmas
	sigma = vsigma
	#print out problem dimensions
	println("Problem has: ")
	println("\t",n," variables")
	println("\t",m," ineq. constraints")
	println("\t",p,"   eq. constraints")
	println()

	#then, decide if there are slacks to be added
	#only add slacks to violated inequality constraints

	xzeros= xzero
	ns = 0

	iSla = true
	iLog = iSla

	Is = Array(Int,0)
	tuttoOK = true
	for i = 1:m
		if g_orig(xzero)[i] <= 0.0 && iSla
			# aggiunge una slack all'i-esimo vincolo
			println("Aggiungo una var. slack al vincolo ",i)
			ns = ns + 1
			Is = [Is; i]
			xzeros = [xzeros; 1.0]
		end
		if g_orig(xzero)[i] <= 0.0
			tuttoOK = false
		end
	end
	if tuttoOK && ~iSla
		iLog = true
	end
	if iSla
		println()
		println(ns," slack variable(s) have been added")
		if ns > 0
			println("one per each of the following constraints")
			for i = 1:ns
				println("\tg_",Is[i],"(x) >= 0  is now: g_",Is[i],"(x) - s",i," = 0, s",i," >= 0")
			end
		end
	end
	println()
	#at this point ns is the number of added slack variables
	#hence transf. problem has: 
	# n+ns variables
	# m inequality constraints (handled by log-barrier)
	# ns equality constraints (handled by seq.penalty)

	#print("\nhit return key to continue")
	#readline()

	f(x) = f_orig(x[1:n])
	function nabla_f(x)
		g1 = zeros(n+ns)
		for i = 1:n
			g1[i] = nabla_f_orig(x[1:n])[i]
		end
		return g1
	end
	function g(x)
		g1 = zeros(m)
		go = g_orig(x[1:n])

		for i = 1:m
			if i in Is
				j = indexin([i],Is)[1]
				g1[i] = x[n+j]
			else
				g1[i] = go[i]
			end
		end 
		return g1
	end
	function nabla_g(x)
		g1 = zeros(n+ns,m)
		g_orig = nabla_g_orig(x[1:n])
		for i = 1:m
			if i in Is
				j = indexin([i],Is)[1]
				g1[n+j,i] = 1.0
			else
				g1[1:n,i] = g_orig[1:n,i]
			end
		end
		return g1
	end
	function h(x)
		g1 = zeros(p+ns)
		g1[1:p] = h_orig(x[1:n])
		go = g_orig(x[1:n])
		for i = 1:m
			if i in Is
				j = indexin([i],Is)[1]
				g1[p+j] = go[i] - x[n+j]
			end
		end
		return g1
	end
	function nabla_h(x)
		g1  = zeros(n+ns,p+ns)
		gh = nabla_h_orig(x[1:n])
		g1[1:n,1:p] = gh
		go = nabla_g_orig(x[1:n])
		for i = 1:m
			if i in Is
				j = indexin([i],Is)[1]
				g1[1:n,p+j] = go[1:n,i]
				g1[n+j,p+j]   = -1.0
			end
		end
		return g1
	end

	rho  = 100.0
	xk    = xzeros
	eta   = 1.0
	gb    = zeros(n+ns)
	la    = zeros(m)
	mu    = zeros(p+ns)

	#define the log-barrier function and its gradient
	function B(x)
		if minimum(g(x)) <= 1.e-10
			return 1.e+10
		else
			temp = f(x)
			for i = 1:m
				temp = temp - rho*log(g(x)[i])
			end
			for i = 1:(p+ns)
				temp = temp + (h(x)[i])^2/(2.0*rho)
			end
			return temp
		end
	end
	function grad_B!(x,gb)
		gp = nabla_f(x) 
		for i = 1:m
			gp = gp - rho*nabla_g(x)[:,i]./(g(x)[i])
		end
		gp = gp + nabla_h(x)*h(x)/rho
		for i = 1:(n+ns)
			gb[i] = gp[i]
		end
	end

	#define sequential Penalty function and its gradient
	function P(x)
		temp = f(x)
		for i = 1:m
			temp = temp + max(0.0,-g(x)[i])^2/(2.0*rho)
		end
		for i = 1:(p+ns)
			temp = temp + (h(x)[i])^2/(2.0*rho)
		end
		return temp
	end
	function grad_P!(x,gb)
		gp = nabla_f(x) 
		for i = 1:m
			gp = gp - nabla_g(x)[:,i]*max(0.0,-g(x)[i])/rho
		end
		gp = gp + nabla_h(x)*h(x)/rho
		for i = 1:(n+ns)
			gb[i] = gp[i]
		end
	end

	#define augmented Lagrangian function and its gradient
	function La(x)
		temp = f(x)
		for i = 1:m
			temp = temp + la[i]*max(-g(x)[i],-rho*la[i]/2.0) + max(-g(x)[i],-rho*la[i]/2.0)^2/rho
		end
		for i = 1:(p+ns)
			temp = temp + mu[i]*h(x)[i] + (h(x)[i])^2/(2.0*rho)
		end
		return temp
	end
	function grad_La!(x,gb)
		gp = nabla_f(x)
		for i = 1:m
			gp = gp - la[i]*nabla_g(x)[:,i] - 2.0*nabla_g(x)[:,i]*max(-g(x)[i],-rho*la[i]/2.0)/rho
		end
		for i = 1:(p+ns)
			gp = gp + nabla_h(x)[:,i]*(mu[i]+h(x)[i]/rho)
		end
		for i = 1:(n+ns)
			gb[i] = gp[i]
		end
	end

	#define KKT violation measure
	function viol_KKT(x)
		gL = nabla_f(x)
		for i = 1:m
			gL = gL - la[i]*nabla_g(x)[:,i]
		end
		for i = 1:(p+ns)
			gL = gL + mu[i]*nabla_h(x)[:,i]
		end
		viol = maximum(abs(gL))
		for i = 1:m
			viol = max(viol,max(-g(x)[i],-la[i]))
		end
		for i = 1:(p+ns)
			viol = max(viol,abs(h(x)[i]))
		end
		return viol
	end

	iP = 0
	iL = 1
	iB = 0

	if iP > 0
		println("\nUse a sequential Penalty...")
		F = P
		grad_F! = grad_P!
	elseif iL > 0 
		println("\nUse an augmented Lagrangian...")
		F = La
		grad_F! = grad_La!
	else
		println("\nUse a log-barrier function...")
		F = B
		grad_F! = grad_B!
	end


	for k = 0:400
		#rho = 10/(0.1*1.1^k)
		rho = rho/1.5
		eta = rho
		for ink = 1:5
			res = optimize(F,grad_F!,xk,g_tol=eta)
			xk = res.minimum
			grad_F!(xk,gb)
			if norm(gb) <= eta
				break
			end
		end	
		#show(res)
		#readline()	
		#aggiornamento dei moltiplicatori
		if iB > 0
			for i = 1:m
				la[i] = rho/g(xk)[i]
			end
		elseif iP > 0
			for i = 1:m
				la[i] = max(0.0,-g(xk)[i])/rho
			end
		else
			for i = 1:m
				la[i] = la[i] + 2.0*max(-g(xk)[i],-rho*la[i]/2.0)/rho
			end
		end
		if iL == 1
			mu = mu+h(xk)./rho
		else
			mu = h(xk)./rho
		end
		grad_F!(xk,gb)
		@printf("%2d %13.6e %13.6e %13.6e %13.6e\n",k,rho,F(xk),norm(gb),viol_KKT(xk))
		if rho < 1.e-6 && norm(gb) < 1.e-5
			break
		end
		if viol_KKT(xk) < 1.e-6
			break
		end
	end
	println()
	xstar = deepcopy(xk)
	#println(xk)
	soglia = 1e-5
	indNZ = Array(Int,0)
	while isempty(indNZ)
		soglia = soglia/2
		for i = 1:n
			if xk[i] <= soglia
				xk[i] = 0.0
			end 
		end
		indNZ = find(xk)
	end
	ii = maximum(indNZ)
	xk[ii] = 1.0 - sum(xk) + xk[ii]
	#println(xk)	
	println("Soluzione primale:")
	for i = 1:n
		@printf("\tx[%2d] = %13.6e\n",i,xk[i])
	end
	if false
		if m > 0
			println("Molt. vincoli di disuguaglianza:")
			for i = 1:m
				@printf("\tla[%2d] = %13.6e\n",i,la[i])
			end
		end
		if p > 0
			println("Molt. vincoli di    uguaglianza:")
			for i = 1:p
				@printf("\tmu[%2d] = %13.6e\n",i,mu[i])
			end
		end
	end
	@printf(fid,"%13.6e %13.6e\n",(xstar[1:n]'*R)[1],(xstar[1:n]'*Q*xstar[1:n])[1])
	@printf(" Return  Variance\n")
	#@printf(    "%13.6e %13.6e\n",(xk[1:n]'*R)[1],(xk[1:n]'*Q*xk[1:n])[1])
	@printf(    "%13.6e %13.6e\n",(xstar[1:n]'*R)[1],(xstar[1:n]'*Q*xstar[1:n])[1])
	flush(fid)
end # for sigma in sigmas

close(fid)


nothing
